from abc import ABC, abstractclassmethod
import pygame
from threading import Thread
from time import time
from pygame import surface
import numpy as np
from pygame import draw
from pygame import mouse
from pygame.transform import scale
from abc import ABC, abstractclassmethod



pygame.font.init()
sysfont = pygame.font.SysFont(pygame.font.get_default_font(), 30)


class GameObject(ABC):
    def init_game(self, game):
        pass

    def draw(self, surface: pygame.Surface):
        pass

    def process_event(self, event: pygame.event.Event):
        pass


class Game():
    width = 1000
    height = 1000
    running = True
    gameObjects: list[GameObject] = []
    dt = 0
    fps = 0

    def __init__(self, width, height) -> None:
        self.width, self.height = width, height
        self.surface = pygame.display.set_mode((self.width, self.height))
        self.screen_size = np.array([width, height])

    def add_gameobject(self, go: GameObject):
        """
        f: function, f(pygame.Surface) -> None
        """
        go.init_game(self)
        self.gameObjects.append(go)

    def _draw_gameObjects(self):
        for go in self.gameObjects:
            go.draw(self.surface)
    
    def _handle_events(self):
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.running = False

            for go in self.gameObjects:
                go.process_event(event)

    def run(self):
        while self.running:
            loop_start = time()

            self._run()

            # calc fps
            self.dt = time() - loop_start
            self.fps = (1 / self.dt) if self.dt else 0

            # draw fps counter
            text = sysfont.render(f"{int(self.fps)} fps", True, (255, 255, 255))
            self.surface.blit(text, text.get_rect().topleft)

            # update screen
            pygame.display.flip()
            self.surface.fill(0)

    def _run(self):
        self._handle_events()
        self._draw_gameObjects()
